#include "device.hpp"

namespace {
    const QString START_DEV_BLOCK = "# Vendors, devices and interfaces";
    const QString START_INTERFACE_BLOCK = "# C class  class_name";
}

/*!
 * \brief Initializes a device object with a path to a file to get the
 * necessary information from.
 * \param[in] fileName Path to the file from which the data is collected.
 */
Device::Device(QString fileName) { parseData(fileName); }

/*!
 * \brief Parses device data from a @p file
 * \param[in] file The file to parse.
 * \param[in] data The string from which the data is parsed.
 */
void Device::parseDevData(QFile& file, QString data="", QString idata="", bool flag=false) {
    if (flag) return;

    QRegExp rx(R"((.{4})\s+\b(.*)\n)");
    if (data.startsWith("\t\t")) {
        rx.indexIn(data);
        _devData[idata].products.top().interfaces.push({rx.cap(1), rx.cap(2)});
    } else if (data.startsWith("\t")) {
        rx.indexIn(data);
        _devData[idata].products.push({rx.cap(1), rx.cap(2)});
    } else if (!data.isEmpty() && data != '\n') {
        rx.indexIn(data);
        idata = rx.cap(1);
        _devData[idata].vendorName = rx.cap(2);
    } else {
        flag = true;
    }

    data = file.readLine();

    parseDevData(file, data, idata, flag);
}

/*!
 * \brief Parses interface data from a @p file
 * \param[in] file The file to parse.
 * \param[in] data The string from which the data is parsed.
 */
void Device::parseInterfaceData(QFile& file, QString data="", QString idata="", bool flag=false) {
    if (flag) return;

    QRegExp rx(R"(([\w\d]{2})\s+\b(.+)\n)");
    if (data.startsWith("\t\t")) {
        rx.indexIn(data);
        _interfaceData[idata].isubclasses.top().iprotocols.push({rx.cap(1), rx.cap(2)});
    } else if (data.startsWith("\t")) {
        rx.indexIn(data);
        _interfaceData[idata].isubclasses.push({rx.cap(1), rx.cap(2)});
    } else if (!data.isEmpty() && data != '\n') {
        rx.indexIn(data);
        idata = rx.cap(1);
        _interfaceData[idata].iclassName = rx.cap(2);
    } else {
        flag = true;
    }

    data = file.readLine();

    parseInterfaceData(file, data, idata, flag);
}

/*!
 * \brief
 * \param[in] file Data collection file
 * \param[in] startsWith The line that starts the block of data.
 * \param[in] rx Pattern to search for the line that the data
 * starts with.
 */ 
QString searchDevEntries(QFile& file, QString startsWith, QRegExp rx) {
    bool flag = false;
    QString lastStr;
    while (!file.atEnd()) {
        QString curLine = file.readLine();
        if (!flag && curLine.startsWith(startsWith)) {
            flag = true;
            continue;
        }

        if (flag && rx.indexIn(curLine)+1 && !rx.cap(0).isEmpty()) {
            lastStr = rx.cap(0);
            break;
        }
    }

    return lastStr;
}

/*!
 * \brief Collects data about devices and interfaces.
 * \param[in] pathToFile The path to the file from which the data is
 * being collected.
 */
void Device::parseData(QString pathToFile) {
    QFile file(pathToFile);
    if (file.open(QIODevice::ReadOnly)) {
        auto firstDevEntry = searchDevEntries(file,
                START_DEV_BLOCK,
                QRegExp(R"((\d{4})\s+(.*))"));
        parseDevData(file, firstDevEntry);

        auto firstInterfaceEntry = searchDevEntries(file,
                START_INTERFACE_BLOCK,
                QRegExp(R"(C\s+([\w\d]{2})\s+(.+))"));
        parseInterfaceData(file, firstInterfaceEntry);

        file.close();
    }
}

/*!
 * \param[in] data Format string "iVendor:iProduct"
 * \return vendor name and product name
 */
QPair<QString, QString> Device::getDevice(QString data) const {
    auto ldata = data.split(":");
    auto iVendor = ldata[0], iproduct = ldata[1];

    QString productName;
    for (auto&& device : _devData[iVendor].products) {
        if (device.product == iproduct) {
            productName = device.productName;
            break;
        }
    }

    return {_devData[iVendor].vendorName, productName};
}

/*!
 * \param[in] data interface class
 * \return the interface class and its name
 */
QPair<QString, QString> Device::getInterface(QString data) const {
    return {data, _interfaceData[data].iclassName};
}
