#include "notifier.hpp"
#include "device.hpp"

#include <QDBusArgument>

#define USB_IDS_PATH "/usr/share/misc/usb.ids"

enum class UsbguardState {
    BLOCK = 1,
    REJECT
};

constexpr const qint32 NOTIFICATION_TIMEOUT = 10000;

/*!
 * \brief A tray notiifcation popup is displayed when a usb device
 * is connected that has a status of "block" or "reject".
 * \param[in] reply The message that the connection sends.
 */
void Notifier::notify(const QDBusMessage& reply) {
    QList<QVariant> args = reply.arguments();

    auto ruleId = args[1].value<quint32>(); // 0 - allow, 1 - block, 2 - reject
    const QDBusArgument arg = args[4].value<QDBusArgument>(); // a{ss}

    QMap<QString, QString> devEntries;

    // get device data
    arg.beginArray();
    while (!arg.atEnd()) {
        QString key, value;

        arg.beginMap();
        arg >> key >> value;

        if (key == "id" || key == "with-interface") {
            devEntries.insert(key, value);
        }

        arg.endMap();
    }
    arg.endArray();

    Device dev(USB_IDS_PATH);
    auto interfaces = devEntries["with-interface"].split(" ");
    QString bInterface = "";

    // search for the device class that is present in usb.ids
    for (auto&& el : interfaces) {
        auto bInterface_tmp = el.split(":")[0];
        if (!dev.getInterface(bInterface_tmp).second.isEmpty()) {
            bInterface = bInterface_tmp;
            break;
        }
    }

    auto [vendorName, devName] = dev.getDevice(devEntries["id"]);
    auto interfaceName = dev.getInterface(bInterface).second;

    // check whether device data is found, otherwise "unknown"
    vendorName = vendorName.isEmpty() ? tr("unknown") : vendorName;
    devName = devName.isEmpty() ? tr("unknown") : devName;
    interfaceName = interfaceName.isEmpty() ? tr("unknown") : interfaceName;

    auto notificationMsgBody = tr("Device: %1, %2")
        .arg(vendorName)
        .arg(devName);
    notificationMsgBody += '\n' + tr("Interface: %1").arg(interfaceName);

    if (ruleId == static_cast<quint32>(UsbguardState::BLOCK) ||
            ruleId == static_cast<quint32>(UsbguardState::REJECT)) {
        QDBusMessage m = QDBusMessage::createMethodCall(
                "org.freedesktop.Notifications",
                "/org/freedesktop/Notifications",
                "org.freedesktop.Notifications",
                "Notify");

        QVariantMap varMap;
        varMap[QLatin1String("urgency")] = QVariant(QLatin1String("<1>"));
        m << "usbguard"                         // app_name
            << quint32(0)                       // replaces_id
            << ""                               // app_icon
            << tr("Blocked by administrator")   // summary
            << notificationMsgBody              // body
            << QStringList("")                  // actions
            << varMap                           // hints
            << NOTIFICATION_TIMEOUT;            // timeout

        if (!QDBusConnection::sessionBus().send(m)) {
            qWarning() << "Failed to send message using \"org.freedesktop.Notifications.Notify\".";
        }
    }
}
